/* VT52 Terminal Enulator										*/

#include <tos.h> 
#include <vdi.h>
#include <aes.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "TERMINAL.H"

#define	TRUE	1
#define	FALSE	0

#define	CON	2
#define ESC 27

typedef int BOOLEAN;

int	work_in[12],work_out[57];

int ap_id;
int p_handle,handle;
int zb,zh,bb,bh;
int m_x,m_y,m_f;

int msg[8];								/* Mitteilungspuffer	*/

OBJECT *menu;

int	nr_speed[16] = {
		BR19200, BR9600, BR4800, BR3600,
		BR2400, BR2000, BR1800, BR1200,
		BR600, BR300, BR200, BR150,
		BR134, BR110, BR75, BR50
	};

int	nr_bits[4] = {BITS8, BITS7, BITS6, BITS5};

BOOLEAN ende;
int		f_handle;
long	bytes;
char	settings_file[14] = "TERMINAL.INF";

struct
	{
	BOOLEAN	halbduplex;
	BOOLEAN	linewrap;
	int		bit8;
	int		interface;
	int		speed;
	BOOLEAN	rts_cts;
	int		ucr;
	} settings, settings1;

/* ------------------------------------------------------------ */


void open_work(void)
	{
	int i;
	ap_id = appl_init();				/* Applikationsnummer	*/
										/* Bildschirm-Handle	*/
	p_handle = graf_handle(&zb,&zh,&bb,&bh);
	for(i = 0; i < 10; i++) work_in[i] = 1;	/* Auf Eins setzen	*/
	work_in[10] = 2;					/* Koordinatensystem	*/
	handle = p_handle;					/* Kopieren				*/
										/* Virtuelle erffnen	*/
	v_opnvwk(work_in ,&handle ,work_out);
	m_x = work_out[0];
	m_y = work_out[1];
	m_f = work_out[39];
	}


void close_work(void)
	{
	v_clsvwk(handle);					/* Virtuelle abmelden	*/
	appl_exit();						/* Anwendung abmelden	*/
	}


/* ------------------------------------------------------------ */


long inchar(void)
	{
	return(Bconin(CON));
	}


void outchar_VT52(int Zeichen)
	{
	Bconout(CON, Zeichen);
	}


void outchar(int Zeichen)
	{
	outchar_VT52(Zeichen & settings.bit8);
	}


void read_char(void)
	{
	while(Bconstat(settings.interface))
		outchar((int)Bconin(settings.interface));
	}


void send_char(int Zeichen)
	{
	BOOLEAN not_sent;

	not_sent = TRUE;
	do
		if(Bcostat(settings.interface))
			{
			Bconout(settings.interface, Zeichen & settings.bit8);
			not_sent = FALSE;
			}
		else
			read_char();
	while(not_sent);
	}


void Terminal_Mode(void)
	{
	long	Zeichen;

	wind_update(BEG_MCTRL);			/* Totale Mauskontrolle		*/
	graf_mouse(M_OFF, 0);			/* Maus aus					*/
	menu_bar(menu, 0);
	form_dial(FMD_START, 0, 0, m_x, m_y, 0, 0, m_x, m_y);
	form_dial(FMD_GROW, m_x/2, m_y/2, 1, 1, 0, 0, m_x, m_y);
	v_enter_cur(handle);
	if(settings.linewrap)
		{
		outchar_VT52(ESC);
		outchar_VT52('v');
		}
	else
		{
		outchar_VT52(ESC);
		outchar_VT52('w');
		}
	printf("******************************\n");
	printf("* VT52 Terminal Emulator 1.1 *\n");
	printf("******************************\n");
	printf("Abbruch mit UNDO\n");
	do
		{
		read_char();
		Zeichen = inchar();
		if(Zeichen & 0xFFL)
			{
			if(settings.halbduplex)
				outchar((int)Zeichen);
			send_char((int)Zeichen);
			}
		}
	while(Zeichen != 0x610000L);
	v_exit_cur(handle);
	form_dial(FMD_FINISH, 0, 0, m_x, m_y, 0, 0, m_x, m_y);
	menu_bar(menu, 1);	
	form_dial(FMD_SHRINK, m_x/2, m_y/2, 1, 1, 0, 0, m_x, m_y);
	graf_mouse(M_ON, 0);
	wind_update(END_MCTRL);
	}


/* ndert den Optionsstatus (TRUE <-> FALSE) sowie dessen Menueintrag	*/
void chg_option(int option, BOOLEAN *mem_option)
	{
	if(*mem_option)
		{
		*mem_option = FALSE;
		menu_icheck(menu, option, 0);
		}
	else
		{
		*mem_option = TRUE;
		menu_icheck(menu, option, 1);
		}
	}


void set_rs232c(void)
	{
	int flowctl;

	if(settings.rts_cts)
		flowctl = 0x2;
	else
		flowctl = 0;
	settings.ucr = (settings.ucr & 0x66) | 0x88;
	Rsconf(settings.speed & 0xF, flowctl, settings.ucr, -1, -1, -1);
	}


void loop(void)
	{
	BOOLEAN	fehler;
	int		hilf, i;

	ende = FALSE;

	settings.halbduplex	= FALSE;
	settings.linewrap	= FALSE;
	settings.bit8		= 0xFF;
	settings.interface	= 1;			/* 1: AUX, 3: MIDI		*/
	settings.speed		= 1;			/* 9600 Baud			*/
	settings.rts_cts	= FALSE;
	settings.ucr		= 0x8A;
	
/*-- Voreinstellungsdatei laden --------------------------------*/

	f_handle = open(settings_file, O_RDONLY);
	if(f_handle != -1)
		{
		bytes = read(f_handle, &settings1, sizeof(settings1));
		close(f_handle);
		if(bytes == sizeof(settings1))
			settings = settings1;
		}

/*-- Einstellungen und Optionen setzen -------------------------*/

	if(settings.halbduplex)
		menu_icheck(menu, HALBDUPL, 1);
	if(settings.linewrap)
		menu_icheck(menu, LNWRAP, 1);
	if((settings.bit8 & 0x80) == 0)
		settings.bit8 = 0x7F;
	else
		{
		settings.bit8 = 0xFF;
		menu_icheck(menu, BIT8ON, 1);
		}
	if(settings.interface == 1)
		menu_icheck(menu, RS232C, 1);
	else
		{
		settings.interface = 3;
		menu_icheck(menu, MIDI, 1);
		}
	settings.speed = settings.speed & 0xF;
	menu_icheck(menu, nr_speed[settings.speed], 1);
	if(settings.rts_cts)
		menu_icheck(menu, RTSCTS, 1);
	hilf = (settings.ucr >> 5) & 0x3;	/* Zeichenlnge ermitteln	*/
	menu_icheck(menu, nr_bits[hilf], 1);
	if((settings.ucr & 0x4) == 0)
		menu_icheck(menu, NOPARIT, 1);
	else
		if(settings.ucr & 0x2)
			menu_icheck(menu, EVEN, 1);
		else
			menu_icheck(menu, ODD, 1);
	set_rs232c();

/*-- Hauptschleife --------------------------------------------*/

	do
		{
		evnt_mesag(msg);
		if(msg[0]==MN_SELECTED)			/* Menu angewhlt		*/
			{
			switch(msg[4])				/* Welches Objekt ?		*/
				{
				case	INFORM:
					form_alert(1,"[1][ | VT52 Terminal Emulator  |      Version 1.1][  OK  ]");
					break;
				case	TMODE:
					Terminal_Mode();
					break;
				case	BEENDEN:
					ende = TRUE;
					break;
				case	HALBDUPL:
					chg_option(HALBDUPL, &settings.halbduplex);
					break;
				case	BIT8ON:
					if((settings.bit8 & 0x80) == 0)
						{
						settings.bit8 = 0xFF;
						menu_icheck(menu, BIT8ON, 1);
						}
					else
						{
						settings.bit8 = 0x7F;
						menu_icheck(menu, BIT8ON, 0);
						}
					break;
				case	LNWRAP:
					chg_option(LNWRAP, &settings.linewrap);
					break;
				case	RS232C:
					settings.interface = 1;
					menu_icheck(menu, RS232C, 1);
					menu_icheck(menu, MIDI, 0);
					break;
				case	MIDI:
					settings.interface = 3;
					menu_icheck(menu, MIDI, 1);
					menu_icheck(menu, RS232C, 0);
					break;
				case	SAVEOPT:
					fehler = FALSE;
					f_handle = creat(settings_file);
					if(f_handle == -1)
						fehler = TRUE;
					else
						{
						bytes = write(f_handle, &settings, sizeof(settings));
						close(f_handle);
						if(bytes != sizeof(settings))
							fehler = TRUE;
						}
					if(fehler)
						form_alert(1,"[2][ | Fehler beim Sichern  | der Optionen!][ Abbruch ]");
					break;
				case	BR19200:
				case	BR9600:
				case	BR4800:
				case	BR2400:
				case	BR2000:
				case	BR1800:
				case	BR1200:
				case	BR600:
				case	BR300:
				case	BR200:
				case	BR150:
				case	BR134:
				case	BR110:
				case	BR75:
				case	BR50:
					for(i=0; i<=15; i++)
						if(msg[4] == nr_speed[i])
							{
							menu_icheck(menu, nr_speed[settings.speed], 0);
							settings.speed = i;
							menu_icheck(menu, nr_speed[i], 1);
							set_rs232c();
							}
					break;
				case	BITS8:
				case	BITS7:
				case	BITS6:
				case	BITS5:
					for(i=0; i<=3; i++)
						if(msg[4] == nr_bits[i])
							{
							hilf = (settings.ucr >> 5) & 0x3;
							menu_icheck(menu, nr_bits[hilf], 0);
							settings.ucr = (settings.ucr & 0x9F) | (i << 5);
							menu_icheck(menu, nr_bits[i], 1);
							set_rs232c();
							}
					break;
				case	NOPARIT:
					menu_icheck(menu, EVEN, 0);
					menu_icheck(menu, ODD, 0);
					settings.ucr = settings.ucr & 0xFB;
					menu_icheck(menu, NOPARIT, 1);
					set_rs232c();
					break;
				case	EVEN:
					menu_icheck(menu, NOPARIT, 0);
					menu_icheck(menu, ODD, 0);
					settings.ucr = settings.ucr | 0x6;
					menu_icheck(menu, EVEN, 1);
					set_rs232c();
					break;
				case	ODD:
					menu_icheck(menu, NOPARIT, 0);
					menu_icheck(menu, EVEN, 0);
					settings.ucr = (settings.ucr & 0xFD) | 0x4;
					menu_icheck(menu, ODD, 1);
					set_rs232c();
					break;
				case	RTSCTS:
					chg_option(RTSCTS, &settings.rts_cts);
					set_rs232c();
					break;
				}
			menu_tnormal(menu,msg[3],1);	/* Invertierung aus */
			}
		}
	while(ende == FALSE);
	}


void main(void)
	{
	open_work();
	if(!rsrc_load("TERMINAL.RSC"))
		form_alert(1,"[2][ | 'TERMINAL.RSC'| nicht gefunden!  ][ Abbruch ]");
	else
		{
		rsrc_gaddr(R_TREE,MENU,&menu);	/* Adresse Objektbaum   */
		menu_bar(menu,1);				/* Menu einschalten     */
		graf_mouse(ARROW,0);			/* Pfeil als Maus		*/
		loop();							/* Hauptschleife		*/
		menu_bar(menu,0);				/* Menu ausschalten     */
		rsrc_free();
		}
	close_work();
	exit(0);
	}
